@@ -16,7 +16,7 @@ from rich.theme import Theme
 from natsort import natsorted, ns
 import regex
 
-VERSION = "1.5.0"
+VERSION = "1.5.4"
 MESSAGE_INDENT = "    "
 UNICODE_BOM = "\ufeff"
 NO_BREAK_SPACE = "\u00a0"
@@ -29,44 +29,12 @@ NO_BREAK_HYPHEN = "\u2011"
 IGNORED_FILENAMES = ["colophon.xhtml", "titlepage.xhtml", "imprint.xhtml", "uncopyright.xhtml", "halftitle.xhtml", "toc.xhtml", "loi.xhtml"]
 # The `re` namespace enables regex functions in xpaths in lxml
 XHTML_NAMESPACES = {"xhtml": "http://www.w3.org/1999/xhtml", "epub": "http://www.idpf.org/2007/ops", "z3998": "http://www.daisy.org/z3998/2012/vocab/structure/", "se": "https://standardebooks.org/vocab/1.0", "dc": "http://purl.org/dc/elements/1.1/", "opf": "http://www.idpf.org/2007/opf", "container": "urn:oasis:names:tc:opendocument:xmlns:container", "m": "http://www.w3.org/1998/Math/MathML", "re": "http://exslt.org/regular-expressions"}
-FRONTMATTER_FILENAMES = ["dedication.xhtml", "introduction.xhtml", "preface.xhtml", "foreword.xhtml", "preamble.xhtml", "titlepage.xhtml", "halftitlepage.xhtml", "imprint.xhtml"]
-BACKMATTER_FILENAMES = ["endnotes.xhtml", "loi.xhtml", "afterword.xhtml", "appendix.xhtml", "colophon.xhtml", "uncopyright.xhtml"]
-BINARY_EXTENSIONS = [".jpg", ".jpeg", ".tif", ".tiff", ".bmp", ".png", ".epub", ".epub3", ".xcf", ".otf"]
-SE_GENRES = ["Adventure", "Autobiography", "Biography", "Childrens", "Comedy", "Drama", "Fantasy", "Fiction", "Horror", "Memoir", "Mystery", "Nonfiction", "Philosophy", "Poetry", "Romance", "Satire", "Science Fiction", "Shorts", "Spirituality", "Tragedy", "Travel"]
-ARIA_ROLES = ["afterword", "appendix", "biblioentry", "bibliography", "chapter", "colophon", "conclusion", "dedication", "epilogue", "foreword", "introduction", "noteref", "part", "preface", "prologue", "subtitle", "toc"]
-IGNORED_CLASSES = ["elision", "name", "temperature", "state", "era", "compass", "acronym", "postal", "eoc", "initialism", "degree", "time", "compound", "timezone", "signature", "full-page"]
 SELECTORS_TO_SIMPLIFY = [":first-child", ":only-child", ":last-child", ":nth-child", ":nth-last-child", ":first-of-type", ":only-of-type", ":last-of-type", ":nth-of-type", ":nth-last-of-type"]
 MESSAGE_TYPE_WARNING = 1
 MESSAGE_TYPE_ERROR = 2
-COVER_TITLE_BOX_Y = 1620 # In px; note that in SVG, Y starts from the TOP of the image
-COVER_TITLE_BOX_HEIGHT = 430
-COVER_TITLE_BOX_WIDTH = 1300
-COVER_TITLE_BOX_PADDING = 100
-COVER_TITLE_MARGIN = 20
-COVER_TITLE_HEIGHT = 80
-COVER_TITLE_SMALL_HEIGHT = 60
-COVER_TITLE_XSMALL_HEIGHT = 50
-COVER_AUTHOR_SPACING = 60
-COVER_AUTHOR_HEIGHT = 40
-COVER_AUTHOR_MARGIN = 20
+COVER_HEIGHT = 2100
+COVER_WIDTH = 1400
 TITLEPAGE_WIDTH = 1400
-TITLEPAGE_VERTICAL_PADDING = 50
-TITLEPAGE_HORIZONTAL_PADDING = 100
-TITLEPAGE_TITLE_HEIGHT = 80 # Height of each title line
-TITLEPAGE_TITLE_MARGIN = 20 # Space between consecutive title lines
-TITLEPAGE_AUTHOR_SPACING = 100 # Space between last title line and first author line
-TITLEPAGE_AUTHOR_HEIGHT = 60 # Height of each author line
-TITLEPAGE_AUTHOR_MARGIN = 20 # Space between consecutive author lines
-TITLEPAGE_CONTRIBUTORS_SPACING = 150 # Space between last author line and first contributor descriptor
-TITLEPAGE_CONTRIBUTOR_DESCRIPTOR_HEIGHT = 40 # Height of each contributor descriptor line
-TITLEPAGE_CONTRIBUTOR_HEIGHT = 40 # Height of each contributor line
-TITLEPAGE_CONTRIBUTOR_MARGIN = 20 # Space between contributor descriptor and contributor line, and between sequential contributor lines
-TITLEPAGE_CONTRIBUTOR_DESCRIPTOR_MARGIN = 80 # Space between last contributor line and next contributor descriptor (if more than one contributor descriptor)
-LEAGUE_SPARTAN_KERNING = 5 # In px
-LEAGUE_SPARTAN_AVERAGE_SPACING = 7 # Guess at average default spacing between letters, in px
-LEAGUE_SPARTAN_100_WIDTHS = {" ": 40.0, "A": 98.245, "B": 68.1875, "C": 83.97625, "D": 76.60875, "E": 55.205, "F": 55.79, "G": 91.57875, "H": 75.0875, "I": 21.98875, "J": 52.631254, "K": 87.83625, "L": 55.205, "M": 106.9, "N": 82.5725, "O": 97.1925, "P": 68.1875, "Q": 98.83, "R": 79.41599, "S": 72.63125, "T": 67.83625, "U": 75.32125, "V": 98.245, "W": 134.62, "X": 101.28625, "Y": 93.1, "Z": 86.19875, ".": 26.78375, ",": 26.78375, "/": 66.08125, "\\": 66.08125, "-": 37.66125, ":": 26.78375, ";": 26.78375, "’": 24.3275, "!": 26.78375, "?": 64.3275, "&": 101.87125, "0": 78.48, "1": 37.895, "2": 75.205, "3": 72.04625, "4": 79.29875, "5": 70.175, "6": 74.26875, "7": 76.95875, "8": 72.16375, "9": 74.26875}
-NOVELLA_MIN_WORD_COUNT = 17500
-NOVEL_MIN_WORD_COUNT = 40000
 RICH_THEME = Theme({
 	"xhtml": "bright_blue",
 	"xml": "bright_blue",
@@ -177,11 +145,11 @@ def print_error(message: Union[SeException, str], verbose: bool = False, is_warn
 	"""
 	Helper function to print a colored error message to the console.
 
-	Allowed BBCode values:
+	Allowed BBCode tags:
 	[link=foo]bar[/] - Hyperlink
 	[xml] - XML, usually a tag
 	[xhtml] - XHTML, usually a tag
-	[attr] - A lone XHTML attribute
+	[attr] - A lone XHTML attribute name (without `="foo"`)
 	[val] - A lone XHTML attribute value (not a class)
 	[class] - A lone XHTML class value
 	[path] - Filesystem path or glob
