@@ -9,19 +9,33 @@ import math
 import string
 import unicodedata
 from pathlib import Path
-from typing import Union
 
 import regex
 import roman
 import tinycss2
 from bs4 import BeautifulSoup, NavigableString, Tag
 from lxml import etree
-#from titlecase import titlecase as pip_titlecase
-from se.vendor.titlecase import titlecase as pip_titlecase
+from titlecase import titlecase as pip_titlecase
+#from se.vendor.titlecase import titlecase as pip_titlecase
 
 import se
 
 
+# This list of phrasing tags is not intended to be exhaustive. The list is only used
+# to resolve the uncommon situation where there is no plain text in a paragraph. The
+# span and br tags are explicitly omitted because of how they are used in poetry formatting,
+# which differs from the normal formatting.
+
+PHRASING_TAGS = [
+	"{http://www.w3.org/1999/xhtml}a",
+	"{http://www.w3.org/1999/xhtml}abbr",
+	"{http://www.w3.org/1999/xhtml}b",
+	"{http://www.w3.org/1999/xhtml}cite",
+	"{http://www.w3.org/1999/xhtml}em",
+	"{http://www.w3.org/1999/xhtml}i",
+	"{http://www.w3.org/1999/xhtml}strong",
+]
+
 def semanticate(xhtml: str) -> str:
 	"""
 	Add semantics to well-formed XHTML
@@ -64,23 +78,28 @@ def semanticate(xhtml: str) -> str:
 	xhtml = regex.sub(r"(?<!\<abbr\>)Inc\.", r"<abbr>Inc.</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)Ltd\.", r"<abbr>Ltd.</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)St\.", r"<abbr>St.</abbr>", xhtml)
-	xhtml = regex.sub(r"(?<!\<abbr\>)MS(S?)\.", r"""<abbr class="initialism">MS\1.</abbr>""", xhtml)
-	xhtml = regex.sub(r"(?<!\<abbr\>)([Vv])iz\.", r"<abbr>\1iz.</abbr>", xhtml)
-	xhtml = regex.sub(r"(\b)(?<!\<abbr\>)etc\.", r"\1<abbr>etc.</abbr>", xhtml)
-	xhtml = regex.sub(r"(\b)(?<!\<abbr\>)([Cc])f\.", r"\1<abbr>\2f.</abbr>", xhtml)
-	xhtml = regex.sub(r"(\b)(?<!\<abbr\>)p\.([\s0-9])", r"\1<abbr>p.</abbr>\2", xhtml)
-	xhtml = regex.sub(r"(\b)(?<!\<abbr\>)ed\.", r"\1<abbr>ed.</abbr>", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Gg])ov\.", r"<abbr>\1ov.</abbr>", xhtml)
+	xhtml = regex.sub(r"(?<!\<abbr\>)Col\.", r"<abbr>Col.</abbr>", xhtml)
+	xhtml = regex.sub(r"(?<!\<abbr\>)MS(S?)\.", r"""<abbr>MS\1.</abbr>""", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Vv])iz\.", r"<abbr>\1iz.</abbr>", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)etc\.", r"<abbr>etc.</abbr>", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Cc])f\.", r"<abbr>\1f.</abbr>", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)p\.([\s0-9])", r"<abbr>p.</abbr>\1", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)ed\.", r"<abbr>ed.</abbr>", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="initialism"\>)([Ii])\.e\.""", r"""<abbr class="initialism">\1.e.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="initialism"\>)([Ee])\.g\.""", r"""<abbr class="initialism">\1.g.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="initialism"\>)\bN\.?B\.\b""", r"""<abbr class="initialism">N.B.</abbr>""", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)(Jan\.|Feb\.|Mar\.|Apr\.|Jun\.|Jul\.|Aug\.|Sep\.|Sept\.|Oct\.|Nov\.|Dec\.)", r"<abbr>\1</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)No\.(\s+[0-9]+)", r"<abbr>No.</abbr>\1", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="degree"\>)PhD""", r"""<abbr class="degree">PhD</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr class="degree"\>)Ph\.?\s*D\.?""", r"""<abbr class="degree">Ph. D.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="initialism"\>)I\.?O\.?U\.?\b""", r"""<abbr class="initialism">I.O.U.</abbr>""", xhtml)
 	xhtml = regex.sub(r"""\b(?<!\<abbr class="era"\>)A\.?D""", r"""<abbr class="era">AD</abbr>""", xhtml)
 	xhtml = regex.sub(r"""\b(?<!\<abbr class="era"\>)B\.?C""", r"""<abbr class="era">BC</abbr>""", xhtml)
 	xhtml = regex.sub(r"""(?<!\<abbr class="time( eoc)?"\>)([ap])\.\s?m\.""", r"""<abbr class="time">\2.m.</abbr>""", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)([Vv])s\.", r"<abbr>\1s.</abbr>", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr class="name"\>)Thos\.""", r"""<abbr class="name">Thos.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr class="name"\>)Jas\.""", r"""<abbr class="name">Jas.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr class="name"\>)Chas\.""", r"""<abbr class="name">Chas.</abbr>""", xhtml)
 
 	# Wrap £sd shorthand
 	xhtml = regex.sub(r"([0-9½¼⅙⅚⅛⅜⅝]+)([sd⅞]\.)", r"\1<abbr>\2</abbr>", xhtml)
@@ -359,21 +378,6 @@ def _indent(tree, space="\t"):
 	else:
 		tree.text = "\n"
 
-# This list of phrasing tags is not intended to be exhaustive. The list is only used
-# to resolve the uncommon situation where there is no plain text in a paragraph. The
-# span and br tags are explicitly omitted because of how they are used in poetry formatting,
-# which differs from the normal formatting.
-
-PHRASING_TAGS = [
-	"{http://www.w3.org/1999/xhtml}a",
-	"{http://www.w3.org/1999/xhtml}abbr",
-	"{http://www.w3.org/1999/xhtml}b",
-	"{http://www.w3.org/1999/xhtml}cite",
-	"{http://www.w3.org/1999/xhtml}em",
-	"{http://www.w3.org/1999/xhtml}i",
-	"{http://www.w3.org/1999/xhtml}strong",
-]
-
 def _indent_children(elem, level, one_space, indentations, has_child_tails=False):
 	"""
 	Recursive helper function implementing indent levels for lxml tree.
@@ -392,7 +396,7 @@ def _indent_children(elem, level, one_space, indentations, has_child_tails=False
 			has_child_tails = True
 		else:
 			for child in elem:
-				if child.tail and not regex.match(r"^\n\s*$", child.tail):
+				if (child.tail and not regex.match(r"^[\n\t ]+$", child.tail)):
 					has_child_tails = True
 					break
 
@@ -416,15 +420,16 @@ def _indent_children(elem, level, one_space, indentations, has_child_tails=False
 
 		next_child = child.getnext()
 
-		# Remove line wraps from child text (except meta tags)
+		# Remove line wraps and extra whitespace from child text (except meta tags)
 		if child.text and not regex.match(r"^[\n\t ]+$", child.text):
 			if child.tag is etree.Comment:
-				child.text = regex.sub(r" *\n\s*", child_indentation, child.text)
+				child.text = regex.sub(r" *\n[\n\t ]*", child_indentation, child.text)
 			elif child.tag != "{http://www.idpf.org/2007/opf}meta":
 				_unwrap_text(child, remove_trailing_space=True)
+				child.text = regex.sub(r"[\t ]+", " ", child.text)
 
 		# Handle different cases for indentation in child tail content
-		if not child.tail or regex.match(r"^\n\s*$", child.tail):
+		if not child.tail or regex.match(r"^[\n\t ]+$", child.tail):
 			if next_child is None:
 				if has_child_tails:
 					child.tail = ""
@@ -440,12 +445,16 @@ def _indent_children(elem, level, one_space, indentations, has_child_tails=False
 			elif not has_child_tails and not child.tail and next_child.tag in PHRASING_TAGS:
 				child.tail = ""
 			elif has_child_tails:
-				child.tail = ""
+				if not child.tail or next_child.tag == "{http://www.w3.org/1999/xhtml}br":
+					child.tail = ""
+				else:
+					child.tail = " "
 			else:
 				child.tail = child_indentation
 		else:
-			# Remove line wraps in child tail
+			# Remove line wraps and extra whitespace in child tail
 			_unwrap_tail(child, remove_trailing_space=next_child is None)
+			child.tail = regex.sub(r"[\t ]+", " ", child.tail)
 			# Add special indentation for br tag with non-empty tail
 			if child.tag == "{http://www.w3.org/1999/xhtml}br":
 				child_indentation = indentations[level - 1]
@@ -455,22 +464,22 @@ def _unwrap_text(elem: etree.Element, remove_trailing_space: bool):
 	"""
 	Remove line wraps from text content of element.
 	"""
-	elem.text = regex.sub(r"^\n\s*", "", elem.text)
+	elem.text = regex.sub(r"^\n[\n\t ]*", "", elem.text)
 	if remove_trailing_space:
-		elem.text = regex.sub(r"\n\s*$", "", elem.text)
-	elem.text = regex.sub(r" *\n\s*", " ", elem.text)
+		elem.text = regex.sub(r"\n[\n\t ]*$", "", elem.text)
+	elem.text = regex.sub(r" *\n[\n\t ]*", " ", elem.text)
 
 def _unwrap_tail(elem: etree.Element, remove_trailing_space: bool):
 	"""
 	Remove line wraps from tail content of element.
 	"""
 	if elem.tag == "{http://www.w3.org/1999/xhtml}br":
-		elem.tail = regex.sub(r"^\n\s*", "", elem.tail)
+		elem.tail = regex.sub(r"^\n[\n\t ]*", "", elem.tail)
 	else:
-		elem.tail = regex.sub(r"^\n\s*", " ", elem.tail)
+		elem.tail = regex.sub(r"^\n[\n\t ]*", " ", elem.tail)
 	if remove_trailing_space:
-		elem.tail = regex.sub(r"\n\s*$", "", elem.tail)
-	elem.tail = regex.sub(r" *\n\s*", " ", elem.tail)
+		elem.tail = regex.sub(r"\n[\n\t ]*$", "", elem.tail)
+	elem.tail = regex.sub(r" *\n[\n\t ]*", " ", elem.tail)
 
 def format_xml_file(filename: Path) -> None:
 	"""
@@ -500,52 +509,102 @@ def format_xml_file(filename: Path) -> None:
 			file.write(processed_xml)
 			file.truncate()
 
-def format_xml(xml: Union[str, etree.ElementTree]) -> str:
+def _format_style_elements(tree: etree.ElementTree):
 	"""
-	Pretty-print well-formed XML.
+	Find <style> elements in an XML etree, and pretty-print the CSS inside of them.
+	The passed tree is modified in-place.
 
 	INPUTS
-	xml: A string of well-formed XML; or an etree of an XML document.
+	tree: An XML etree.
 
 	OUTPUTS
-	A string of pretty-printed XML.
+	None.
 	"""
 
-	# Canonicalize and format XML
 	try:
-		if isinstance(xml, str):
-			tree = etree.fromstring(str.encode(xml))
-		else:
-			tree = xml
+		for node in tree.xpath("//svg:style", namespaces={"xhtml": "http://www.w3.org/1999/xhtml", "svg": "http://www.w3.org/2000/svg"}):
+			css = format_css(node.text)
 
-		canonical_bytes = etree.tostring(tree, method="c14n")
-		tree = etree.fromstring(canonical_bytes)
-		_indent(tree, space="\t")
-		xml = """<?xml version="1.0" encoding="utf-8"?>\n""" + etree.tostring(tree, encoding="unicode") + "\n"
+			# Get the <style> element's indentation
+			indent = node.xpath("preceding-sibling::text()[1]")[0].replace("\n", "")
 
-	except Exception as ex:
-		raise se.InvalidXmlException(f"Couldn’t parse XML file. Exception: {ex}")
-
-	# Normalize unicode characters
-	xml = unicodedata.normalize("NFC", xml)
-
-	# Almost done. Let's clean CSS in <style> elements, like we find in SVG files.
-	try:
-		matches = regex.findall(r"^(\s*)(<style[^>]*?>)(.+?)(</style>)", xml, flags=regex.DOTALL | regex.MULTILINE)
-		for match in matches:
-			css = format_css(match[2])
 			# Indent the CSS one level deeper than the <style> element
-			css = ''.join(match[0] + "\t" + line + "\n" for line in css.splitlines())
+			css = ''.join(indent + "\t" + line + "\n" for line in css.splitlines())
 			css = css.strip("\n")
 			css = regex.sub(r"^\s+$", "", css, flags=regex.MULTILINE) # Remove indents from lines that are just white space
-			xml = xml.replace(f"{match[0]}{match[1]}{match[2]}{match[3]}", f"{match[0]}{match[1]}\n{css}\n{match[0]}{match[3]}")
+
+			node.text = "\n" + css + "\n" + indent
 	except se.InvalidCssException as ex:
 		raise ex
 	except Exception as ex:
 		raise se.InvalidCssException(f"Couldn’t parse CSS. Exception: {ex}")
 
+def _format_xml_str(xml: str) -> etree.ElementTree:
+	"""
+	Given a string of well-formed XML, return a pretty-printed etree.
+
+	INPUTS
+	xml: A string of well-formed XML.
+
+	OUTPUTS
+	An etree representing the pretty-printed XML.
+	"""
+
+	tree = etree.fromstring(str.encode(xml))
+	canonical_bytes = etree.tostring(tree, method="c14n")
+	tree = etree.fromstring(canonical_bytes)
+	_indent(tree, space="\t")
+
+	# Remove white space around attribute values
+	for node in tree.xpath("//*[attribute::*[re:test(., '^\\s+') or re:test(., '\\s+$')]]", namespaces={"re": "http://exslt.org/regular-expressions"}):
+		for attribute in node.keys():
+			value = node.get(attribute)
+			value = regex.sub(r"^\s+", "", value)
+			value = regex.sub(r"\s+$", "", value)
+			node.set(attribute, value)
+
+	return tree
+
+def _xml_tree_to_string(tree: etree.ElementTree, doctype: str = None) -> str:
+	"""
+	Given an XML etree, return a string representing the etree's XML.
+
+	INPUTS
+	tree: An XML etree.
+
+	OUTPUTS
+	A string representing the etree's XML.
+	"""
+
+	xml = """<?xml version="1.0" encoding="utf-8"?>\n""" + etree.tostring(tree, encoding="unicode", doctype=doctype) + "\n"
+
+	# Normalize unicode characters
+	xml = unicodedata.normalize("NFC", xml)
+
 	return xml
 
+def format_xml(xml: str) -> str:
+	"""
+	Pretty-print well-formed XML.
+
+	INPUTS
+	xml: A string of well-formed XML.
+
+	OUTPUTS
+	A string of pretty-printed XML.
+	"""
+
+	try:
+		tree = _format_xml_str(xml)
+	except Exception as ex:
+		raise se.InvalidXmlException(f"Couldn’t parse XML file. Exception: {ex}")
+
+
+	# Pull out the doctype if there is one, as etree seems to eat it
+	doctypes = regex.search(r"<!doctype[^>]+?>", xml, flags=regex.IGNORECASE)
+
+	return _xml_tree_to_string(tree, doctypes.group(0) if doctypes else None)
+
 def format_xhtml(xhtml: str) -> str:
 	"""
 	Pretty-print well-formed XHTML.
@@ -571,7 +630,7 @@ def format_xhtml(xhtml: str) -> str:
 	xhtml = regex.sub(r"([^\s>])\s+(</[^>]+?>)", r"\1\2", xhtml, flags=regex.IGNORECASE)
 
 	try:
-		tree = etree.fromstring(str.encode(xhtml))
+		tree = _format_xml_str(xhtml)
 	except Exception as ex:
 		raise se.InvalidXhtmlException(f"Couldn’t parse XHTML file. Exception: {ex}")
 
@@ -585,14 +644,11 @@ def format_xhtml(xhtml: str) -> str:
 	for node in tree.xpath("//*[re:test(local-name(), '[A-Z]')]", namespaces=se.XHTML_NAMESPACES):
 		node.tag = node.tag.lower()
 
-	# Canonicalize and format XHTML
-	try:
-		xhtml = format_xml(tree)
-	except Exception as ex:
-		raise se.InvalidXhtmlException(f"Couldn’t parse file. Files must be in XHTML format, which is not the same as HTML. Exception: {ex}")
+	# Format <style> elements
+	_format_style_elements(tree)
 
 	# Remove white space between non-tags and <br/>
-	xhtml = regex.sub(r"([^>\s])\s+<br/>", r"\1<br/>", xhtml)
+	xhtml = regex.sub(r"([^>\s])\s+<br/>", r"\1<br/>", _xml_tree_to_string(tree))
 
 	return xhtml
 
@@ -608,27 +664,40 @@ def format_opf(xml: str) -> str:
 	"""
 
 	# Replace html entities in the long description so we can clean it too.
-	# We re-establish them later.
+	# We re-establish them later. Don't use html.unescape because that will unescape
+	# things like &amp; which would make an invalid XML document. (&amp; may appear in translator info,
+	# or other parts of the metadata that are not the long description.
 	xml = xml.replace("&lt;", "<")
 	xml = xml.replace("&gt;", ">")
+	xml = xml.replace("&amp;amp;", "&amp;") # Unescape escaped ampersands, which appear in the long description only
 
 	# Canonicalize and format XML
 	try:
-		xml = format_xml(xml)
+		tree = _format_xml_str(xml)
 	except Exception as ex:
 		raise se.InvalidXmlException(f"Couldn’t parse OPF file. Exception: {ex}")
 
-	# Clean the long description, if we can find it
-	xml = xml.replace(" &lt;p&gt;", "\n\t\t\t&lt;p&gt;")
-	xml = xml.replace("&lt;/p&gt; </meta>", "&lt;/p&gt;\n\t\t</meta>")
+	# Format the long description, then escape it
+	for node in tree.xpath("/opf:package/opf:metadata/opf:meta[@property='se:long-description']", namespaces={"opf": "http://www.idpf.org/2007/opf"}):
+		# Convert the node contents to escaped text.
+		xhtml = node.text # This preserves the initial newline and indentation
 
-	long_description = regex.findall(r"<meta id=\"long-description\" property=\"se:long-description\" refines=\"#description\">(.+?)</meta>", xml, flags=regex.DOTALL)
-	if long_description:
-		escaped_long_description = long_description[0].replace("<", "&lt;")
-		escaped_long_description = escaped_long_description.replace(">", "&gt;")
-		xml = xml.replace(long_description[0], escaped_long_description)
+		if xhtml is None:
+			xhtml = ""
 
-	return xml
+		for child in node:
+			xhtml += etree.tostring(child, encoding="unicode")
+
+		# After composing the string, lxml adds namespaces to every tag. The only way to remove them is with regex.
+		xhtml = regex.sub(r"\sxmlns(:.+?)?=\"[^\"]+?\"", "", xhtml)
+
+		# Remove the children so that we can replace them with the escaped xhtml
+		for child in node:
+			node.remove(child)
+
+		node.text = xhtml
+
+	return _xml_tree_to_string(tree)
 
 def format_svg(svg: str) -> str:
 	"""
@@ -642,22 +711,22 @@ def format_svg(svg: str) -> str:
 	"""
 
 	try:
-		tree = etree.fromstring(str.encode(svg))
-
-		# Make sure viewBox is correctly-cased
-		for node in tree.xpath("/svg:svg", namespaces={"svg": "http://www.w3.org/2000/svg"}):
-			for key, value in node.items(): # Iterate over attributes
-				if key.lower() == "viewbox":
-					node.attrib.pop(key) # Remove the attribute
-					node.attrib["viewBox"] = value # Re-add the attribute, correctly-cased
-					break
-
-		svg = format_xml(tree)
-
+		tree = _format_xml_str(svg)
 	except Exception as ex:
 		raise se.InvalidXmlException(f"Couldn’t parse SVG file. Exception: {ex}")
 
-	return svg
+	# Make sure viewBox is correctly-cased
+	for node in tree.xpath("/svg:svg", namespaces={"svg": "http://www.w3.org/2000/svg"}):
+		for key, value in node.items(): # Iterate over attributes
+			if key.lower() == "viewbox":
+				node.attrib.pop(key) # Remove the attribute
+				node.attrib["viewBox"] = value # Re-add the attribute, correctly-cased
+				break
+
+	# Format <style> elements
+	_format_style_elements(tree)
+
+	return _xml_tree_to_string(tree)
 
 def _format_css_component_list(content: list, in_selector=False, in_paren_block=False) -> str:
 	"""
@@ -1184,7 +1253,7 @@ def generate_title(xhtml: str) -> str:
 									title += span_child
 						else:
 							title = h_child.text
-				elif isinstance(h_child, Tag) and h_child.name == "abbr":
+				elif isinstance(h_child, Tag) and (h_child.name == "abbr" or h_child.name == "i"):
 					title += h_child.text
 				elif isinstance(h_child, NavigableString) and not h_child.isspace():
 					title += h_child
