@@ -783,6 +783,9 @@ class SeEpub:
 		if "halftitle.xhtml" in filenames:
 			spine.append("<itemref idref=\"halftitle.xhtml\"/>")
 
+		if "prologue.xhtml" in filenames:
+			spine.append("<itemref idref=\"prologue.xhtml\"/>")
+
 		for filename in filenames:
 			if filename not in excluded_files:
 				spine.append(f"<itemref idref=\"{filename}\"/>")
@@ -822,8 +825,7 @@ class SeEpub:
 		list of content files in the order given in the spine in content.opf
 		"""
 
-		return regex.findall(r"<itemref idref=\"(.*?)\"/>", self.metadata_xml)
-
+		return self.metadata_dom.xpath("/package/spine/itemref/@idref")
 
 	def get_work_type(self) -> str:
 		"""
@@ -837,17 +839,20 @@ class SeEpub:
 		"""
 
 		worktype = "fiction"  # default
-		subjects = regex.findall(r"<meta property=\"se:subject\">([^<]+?)</meta>", self.metadata_xml)
+
+		subjects = self.metadata_dom.xpath("/package/metadata/meta[@property='se:subject']/text()")
 		if not subjects:
 			return worktype
 
 		# Unfortunately, some works are tagged "Philosophy" but are nevertheless fiction, so we have to double-check
 		if "Nonfiction" in subjects:
 			return "non-fiction"
-		nonfiction_types = ["Adventure", "Autobiography", "Memoir", "Philosophy", "Spirituality", "Travel"]
+
+		nonfiction_types = ["Autobiography", "Memoir", "Philosophy", "Spirituality", "Travel"]
 		for nonfiction_type in nonfiction_types:
 			if nonfiction_type in subjects:
 				worktype = "non-fiction"
+
 		fiction_types = ["Fantasy", "Fiction", "Horror", "Mystery", "Science Fiction"]
 		for fiction_type in fiction_types:
 			if fiction_type in subjects:
