@@ -35,12 +35,11 @@ from se.vendor.kobo_touch_extended import kobo
 from se.vendor.mobi import mobi
 
 
-COVER_SVG_WIDTH = 1400
-COVER_SVG_HEIGHT = 2100
-COVER_THUMBNAIL_WIDTH = int(COVER_SVG_WIDTH / 4) # Cast to int required for PIL
-COVER_THUMBNAIL_HEIGHT = int(COVER_SVG_HEIGHT / 4) # Cast to int required for PIL
+COVER_THUMBNAIL_WIDTH = int(se.COVER_WIDTH / 4) # Cast to int required for PIL
+COVER_THUMBNAIL_HEIGHT = int(se.COVER_HEIGHT / 4) # Cast to int required for PIL
 SVG_OUTER_STROKE_WIDTH = 2
 SVG_TITLEPAGE_OUTER_STROKE_WIDTH = 4
+ARIA_ROLES = ["afterword", "appendix", "biblioentry", "bibliography", "chapter", "colophon", "conclusion", "dedication", "epilogue", "foreword", "introduction", "noteref", "part", "preface", "prologue", "subtitle", "toc"]
 
 def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, output_directory: Path, proof: bool, build_covers: bool) -> None:
 	"""
@@ -89,22 +88,20 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 		except:
 			raise se.InvalidSeEbookException(f"Missing [xml]<dc:identifier>[/] element in [path][link=file://{self.metadata_file_path}]{self.metadata_file_path}[/][/].")
 
-		try:
-			title = self.metadata_dom.xpath("//dc:title")[0].inner_xml()
-			url_title = se.formatting.make_url_safe(title)
-		except:
+		if not self.metadata_dom.xpath("//dc:title"):
 			raise se.InvalidSeEbookException(f"Missing [xml]<dc:title>[/] element in [path][link=file://{self.metadata_file_path}]{self.metadata_file_path}[/][/].")
 
+		output_filename = identifier.replace("https://standardebooks.org/ebooks/", "").replace("/", "_")
 		url_author = ""
 		for author in self.metadata_dom.xpath("//dc:creator"):
 			url_author = url_author + se.formatting.make_url_safe(author.inner_xml()) + "_"
 
 		url_author = url_author.rstrip("_")
 
-		epub_output_filename = f"{url_author}_{url_title}{'.proof' if proof else ''}.epub"
-		epub3_output_filename = f"{url_author}_{url_title}{'.proof' if proof else ''}.epub3"
-		kobo_output_filename = f"{url_author}_{url_title}{'.proof' if proof else ''}.kepub.epub"
-		kindle_output_filename = f"{url_author}_{url_title}{'.proof' if proof else ''}.azw3"
+		epub_output_filename = f"{output_filename}{'.proof' if proof else ''}.epub"
+		epub3_output_filename = f"{output_filename}{'.proof' if proof else ''}.epub3"
+		kobo_output_filename = f"{output_filename}{'.proof' if proof else ''}.kepub.epub"
+		kindle_output_filename = f"{output_filename}{'.proof' if proof else ''}.azw3"
 
 		# Clean up old output files if any
 		se.quiet_remove(output_directory / f"thumbnail_{asin}_EBOK_portrait.jpg")
@@ -339,7 +336,9 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 		metadata_xml = metadata_xml.replace(".svg", ".png")
 		metadata_xml = metadata_xml.replace("id=\"cover.jpg\" media-type=\"image/svg+xml\"", "id=\"cover.jpg\" media-type=\"image/jpeg\"")
 		metadata_xml = metadata_xml.replace("image/svg+xml", "image/png")
-		metadata_xml = regex.sub(r"properties=\"([^\"]*?)svg([^\"]*?)\"", "properties=\"\\1\\2\"", metadata_xml) # We may also have the `mathml` property
+		metadata_xml = regex.sub(r" properties=\"([^\"]*?)svg([^\"]*?)\"", r''' properties="\1\2"''', metadata_xml) # We may also have the `mathml` property
+		metadata_xml = regex.sub(r" properties=\"([^\s]*?)\s\"", r''' properties="\1"''', metadata_xml) # Clean up trailing white space in property attributes introduced by the above line
+		metadata_xml = regex.sub(r" properties=\"\s*\"", "", metadata_xml) # Remove any now-empty property attributes
 
 		# Add an element noting the version of the se tools that built this ebook
 		metadata_xml = regex.sub(r"<dc:publisher", f"<meta property=\"se:built-with\">{se.VERSION}</meta>\n\t\t<dc:publisher", metadata_xml)
@@ -407,7 +406,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 					# Convert SVGs to PNGs at 2x resolution
 					# Path arguments must be cast to string
-					svg2png(url=str(filename), write_to=regex.sub(r"\.svg$", ".png", str(filename)), scale=2)
+					svg2png(url=str(filename), write_to=str(filename.parent / (str(filename.stem) + ".png")), scale=2)
 					(filename).unlink()
 
 				if filename.suffix == ".xhtml":
@@ -435,7 +434,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 							mathml_presentation_xhtml = etree.tostring(mathml_presentation_tree, encoding="unicode", pretty_print=True, with_tail=False).strip()
 
 							# Plop our string back in to the XHTML we're processing
-							processed_xhtml = regex.sub(r"<math[^>]*?>\{}\</math>".format(regex.escape(line)), mathml_presentation_xhtml, processed_xhtml, flags=regex.MULTILINE)
+							processed_xhtml = regex.sub(r"<(?:m:)?math[^>]*?>\{}\</(?:m:)?math>".format(regex.escape(line)), mathml_presentation_xhtml, processed_xhtml, flags=regex.MULTILINE)
 
 						if filename.name == "endnotes.xhtml":
 							# iOS renders the left-arrow-hook character as an emoji; this fixes it and forces it to render as text.
@@ -443,8 +442,13 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 							# See http://mts.io/2015/04/21/unicode-symbol-render-text-emoji/
 							processed_xhtml = processed_xhtml.replace("\u21a9", "\u21a9\ufe0e")
 
+						# Since we added an outlining stroke to the titlepage/publisher logo images, we
+						# want to remove the se:color-depth.black-on-transparent semantic
+						if filename.name in ("colophon.xhtml", "imprint.xhtml", "titlepage.xhtml"):
+							processed_xhtml = regex.sub(r"\s*se:color-depth\.black-on-transparent\s*", "", processed_xhtml)
+
 						# Add ARIA roles, which are just mostly duplicate attributes to epub:type
-						for role in se.ARIA_ROLES:
+						for role in ARIA_ROLES:
 							processed_xhtml = regex.sub(fr"(epub:type=\"[^\"]*?{role}[^\"]*?\")", f"\\1 role=\"doc-{role}\"", processed_xhtml)
 
 						# Some ARIA roles can't apply to some elements.
@@ -461,14 +465,6 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						# See The Man Who Was Thursday by G K Chesterton
 						processed_xhtml = regex.sub(r"(<h[1-6] [^>]*) role=\".*?\">", "\\1>", processed_xhtml)
 
-						# Since we convert SVGs to raster, here we add the color-depth semantic for night mode
-						processed_xhtml = processed_xhtml.replace("z3998:publisher-logo", "z3998:publisher-logo se:image.color-depth.black-on-transparent")
-						processed_xhtml = regex.sub(r"class=\"([^\"]*?)epub-type-z3998-publisher-logo([^\"]*?)\"", "class=\"\\1epub-type-z3998-publisher-logo epub-type-se-image-color-depth-black-on-transparent\\2\"", processed_xhtml)
-
-						# Special case for the titlepage
-						if filename.name == "titlepage.xhtml":
-							processed_xhtml = processed_xhtml.replace("<img", "<img class=\"epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\"")
-
 						# Google Play Books chokes on https XML namespace identifiers (as of at least 2017-07)
 						processed_xhtml = processed_xhtml.replace("https://standardebooks.org/vocab/1.0", "http://standardebooks.org/vocab/1.0")
 
@@ -501,6 +497,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 
 						# Some minor code style cleanup
 						processed_xhtml = processed_xhtml.replace(" >", ">")
+						processed_xhtml = regex.sub(r"""\s*epub:type=""\s*""", "", processed_xhtml)
 
 						if processed_xhtml != xhtml:
 							file.seek(0)
@@ -584,6 +581,11 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 							file.write(xhtml)
 							file.truncate()
 
+				# All done, clean the output
+				# Note that we don't clean .xhtml files, because the way kobo spans are added means that it will screw up spaces inbetween endnotes.
+				for filepath in se.get_target_filenames([kobo_work_directory], (".svg", ".opf", ".ncx")):
+					se.formatting.format_xml_file(filepath)
+
 				se.epub.write_epub(kobo_work_directory, output_directory / kobo_output_filename)
 
 		# Now work on more epub2 compatibility
@@ -632,7 +634,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 								# Check if there's MathML we want to convert
 								# We take a naive approach and use some regexes to try to simplify simple MathML expressions.
 								# For each MathML expression, if our round of regexes finishes and there is still MathML in the processed result, we abandon the attempt and render to PNG using Firefox.
-								for line in regex.findall(r"<(?:m:)math[^>]*?>(?:.+?)</(?:m:)math>", processed_xhtml, flags=regex.DOTALL):
+								for line in regex.findall(r"<(?:m:)?math[^>]*?>(?:.+?)</(?:m:)?math>", processed_xhtml, flags=regex.DOTALL):
 									if line not in replaced_mathml:
 										replaced_mathml.append(line) # Store converted lines to save time in case we have multiple instances of the same MathML
 										mathml_tree = se.easy_xml.EasyXhtmlTree("<?xml version=\"1.0\" encoding=\"utf-8\"?>{}".format(regex.sub(r"<(/?)m:", "<\\1", line)))
@@ -664,9 +666,9 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 										# Did we succeed? Is there any more MathML in our string?
 										if regex.findall("</?(?:m:)?m", processed_line):
 											# Failure! Abandon all hope, and use Firefox to convert the MathML to PNG.
-											se.images.render_mathml_to_png(driver, regex.sub(r"<(/?)m:", "<\\1", line), work_epub_root_directory / "epub" / "images" / f"mathml-{mathml_count}.png")
+											se.images.render_mathml_to_png(driver, regex.sub(r"<(/?)m:", "<\\1", line), work_epub_root_directory / "epub" / "images" / f"mathml-{mathml_count}.png", work_epub_root_directory / "epub" / "images" / f"mathml-{mathml_count}-2x.png")
 
-											processed_xhtml = processed_xhtml.replace(line, f"<img class=\"mathml epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\" src=\"../images/mathml-{mathml_count}.png\" />")
+											processed_xhtml = processed_xhtml.replace(line, f"<img class=\"mathml epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\" src=\"../images/mathml-{mathml_count}.png\" srcset=\"../images/mathml-{mathml_count}-2x.png 2x, ../images/mathml-{mathml_count}.png 1x\" />")
 											mathml_count = mathml_count + 1
 										else:
 											# Success! Replace the MathML with our new string.
